import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import javax.swing.SwingWorker;

import org.apache.commons.net.ftp.FTP;
import org.apache.commons.net.ftp.FTPClient;


public class DatabaseDownload extends SwingWorker<Void, Void>{
	String product = null;
	String downloadMode = null;//What should be downloaded
	String option =null;//How it should be downloaded
	String lastDay =null, lastFile = null, lastYear = null;
	
	DatabaseDownload(String product, String downloadMode, String option, String lastDay, String lastFile, String lastYear){
		this.product = product;
		this.downloadMode = downloadMode;
		this.option = option;
		this.lastDay = lastFile;
		this.lastFile = lastDay;
		this.lastYear = lastYear;
	}
	
	//A method that downloads/updates the data from the ftp server for a determined Modis products
	//Download Mode: 0 - All, 1-Just Metadata, 2- Just HDF
	//It only download the file if it doesn't already exist in the download folder,
	//this way its useful to update the database more efficiently
	@Override
	protected Void doInBackground() throws Exception {

		FTPClient ftpClient2 = new FTPClient();
		//List of ftpDownloads that contains every file of the selected product ready to download from the ftp server
		List<FtpDownload> downloads = new ArrayList<FtpDownload>();
		String[] daysAvailable = null;
		String[] filesAvailable = null;

		try {

			//Creates the connection to the Modis FTP Server
			ftpClient2.connect(FtpDownload.modisFtpServer);
			ftpClient2.login(FtpDownload.modisServerUsername, FtpDownload.modisServerPassword);
			ftpClient2.enterLocalPassiveMode();
			ftpClient2.setFileType(FTP.BINARY_FILE_TYPE);

			
		}catch (IOException e){
			
			e.printStackTrace();
		}
		//For each Modis terra products...
			
		try {
			daysAvailable = ftpClient2.listNames("allData/1/".concat(product).concat("/"+MADGui.getYear()+"/"));
			
			
		}catch (IOException e){
			
			e.printStackTrace();
		}
		
		
		
			//For each day...
			for(int j=0; j< daysAvailable.length; j++){
				
				
				//If the option of only future files is selected and the day is minor than the day the option was selected
				if(option == "ONLY_FUTURE_FILES" && Integer.parseInt(daysAvailable[j]) < Integer.parseInt(lastDay) && MADGui.getYear().equals(lastYear)){
					
					continue;
					
				}

				try {
					filesAvailable = ftpClient2.listNames("allData/1/".concat(product).concat("/"+MADGui.getYear()+"/"+daysAvailable[j]));
					
					

					
					if(filesAvailable.equals(null)){
		
						continue;
						
					}
					
					else if(filesAvailable.length == 0){

						continue;	
					}
				
				}catch (IOException e){
					
					e.printStackTrace();
				}
				
				File folder = new File(MADGui.staticLocalFilePath +"/"+product);
				folder.mkdirs();//This will create the directories if they doesn't exist already
				
				//List of the files already in the download folder of the user
				File[] listOfFiles = folder.listFiles();
				boolean checkExistence = false;
				int lastIndex = -1;
				
				//For each file...
				//Downloads All files (metadata and the HDF)
				if(downloadMode == "ALL_FILES"){
					
					for(int k = 0; k< filesAvailable.length; k++){
						
						checkExistence = false;
						lastIndex = -1;
						
						for(int i=0; i< filesAvailable.length; i++){
							//This is to check for the index of the last file if it exist
							if(option=="ONLY_FUTURE_FILES" && filesAvailable[i].toString().contains(lastFile)){
								lastIndex = i;
							}
						}
						
						for(int i=0; i < listOfFiles.length; i++ ){
							
							//Check if the file already exist
							if( listOfFiles[i].toString().contains(filesAvailable[k]) ){
							
								checkExistence = true;
								break;
							
							}
						}
						//If the file doesn't exist, then its added to the download list
						if(!checkExistence){
							if(option == "ONLY_FUTURE_FILES" && k > lastIndex && Integer.parseInt(MADGui.getYear()) >= Integer.parseInt(lastYear)){
								downloads.add(new FtpDownload(null,"allData/1/"+ product +"/"+ MADGui.getYear()+"/"+daysAvailable[j]+"/"+filesAvailable[k],MADGui.staticLocalFilePath +"/"+product+"/"+ filesAvailable[k],false));
							}
							
							if(option == "ALL_PAST_FILES"){
							downloads.add(new FtpDownload(null,"allData/1/"+ product +"/"+ MADGui.getYear()+"/"+daysAvailable[j]+"/"+filesAvailable[k],MADGui.staticLocalFilePath +"/"+product+"/"+ filesAvailable[k],false));
							}
						}		
							
					}
					
				}
				
				//Download just the metadata for that product
				if(downloadMode == "METADATA_ONLY"){	
					
					for(int k = 0; k< filesAvailable.length; k++){

						if(filesAvailable[k].contains(".met")){
							
							checkExistence = false;
							lastIndex =-1;

							for(int i=0; i< filesAvailable.length; i++){
								//This is to check for the index of the last file if it exist
								if(option=="ONLY_FUTURE_FILES" && filesAvailable[i].toString().contains(lastFile)){
									lastIndex = i;
								}
							}
							
							for(int i=0; i < listOfFiles.length; i++ ){
					
								//Check if the file already exist
								if( listOfFiles[i].toString().contains(filesAvailable[k]) ){
									
									checkExistence = true;

									break;
								
								}
							}
							//If the file doesn't exist, then its added to the download list
							if(!checkExistence){
								if(option == "ONLY_FUTURE_FILES" && k > lastIndex && Integer.parseInt(MADGui.getYear()) >= Integer.parseInt(lastYear)){

									downloads.add(new FtpDownload(null,"allData/1/"+ product +"/"+ MADGui.getYear()+"/"+daysAvailable[j]+"/"+filesAvailable[k],MADGui.staticLocalFilePath +"/"+product+"/"+ filesAvailable[k],false));
								}
								
								if(option == "ALL_PAST_FILES"){
									downloads.add(new FtpDownload(null,"allData/1/"+ product +"/"+ MADGui.getYear()+"/"+daysAvailable[j]+"/"+filesAvailable[k],MADGui.staticLocalFilePath +"/"+product+"/"+ filesAvailable[k],false));
								}
							}		
							
						}
					}
				}
				
				//Downloads the HDFs
				else{
					
					for(int k = 0; k< filesAvailable.length; k++){
						
						if(!filesAvailable[k].contains(".met")){
							
							checkExistence = false;
							lastIndex = -1;
							
							for(int i=0; i< filesAvailable.length; i++){
								//This is to check for the index of the last file if it exist
								if(option=="ONLY_FUTURE_FILES" && filesAvailable[i].toString().contains(lastFile)){
									lastIndex = i;
								}
							}
							
							for(int i=0; i < listOfFiles.length; i++ ){
							
								//Check if the file already exist
								if( listOfFiles[i].toString().contains(filesAvailable[k]) ){
									
									checkExistence = true;
									
									break;
									
								}
							}
							//If the file doesn't exist, then its added to the download list
							if(!checkExistence){
								
								if(option == "ONLY_FUTURE_FILES" && k > lastIndex && Integer.parseInt(MADGui.getYear()) >= Integer.parseInt(lastYear)){
									downloads.add(new FtpDownload(null,"allData/1/"+ product +"/"+ MADGui.getYear()+"/"+daysAvailable[j]+"/"+filesAvailable[k],MADGui.staticLocalFilePath +"/"+product+"/"+ filesAvailable[k],false));
								}
								
								if(option == "ALL_PAST_FILES"){
								downloads.add(new FtpDownload(null,"allData/1/"+ product +"/"+ MADGui.getYear()+"/"+daysAvailable[j]+"/"+filesAvailable[k],MADGui.staticLocalFilePath +"/"+product+"/"+ filesAvailable[k],false));
								}
							}		
							
						}
					}
				}										
				
				//The actuals downloads
				for(int i=0; i < downloads.size(); i++){
					
					downloads.get(i).execute();
					
				}
				
			}
		
		return null;
	}
	
	//Overrides the "Done" methods, that executes a command on the EDT. In this case shows a dialog.
		@Override
	    protected void done() {
	        
	            System.out.println("Database Update: Started");
	                    
	    }

}
