//FTP Library
import org.apache.commons.net.ftp.FTP;
import org.apache.commons.net.ftp.FTPClient;
import org.apache.commons.net.ftp.FTPFile;





//IO
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;



import java.util.ArrayList;
import java.util.List;


//Some GUI
import javax.swing.JOptionPane;
//GUI-Thread Library
import javax.swing.SwingWorker;
 
/**
 * Execute file download from the Modis FTP Server in a background thread while updates the progress.
 * */

public class FtpDownload extends SwingWorker<Void, Void> {
 
    String ftpFilePath;
    String localFilePath;
    FTPClient ftpClient;
    MADGui madGui;
    InputStream inputStream;
    boolean verbose;
    
    static List<FtpDownload> ftpDownloadsList = new ArrayList<FtpDownload>();
    
    //FTP Connection Strings
  	final static String modisFtpServer = "nrt1.modaps.eosdis.nasa.gov";
  	final static String backupftpserver = "nrt2.modaps.eosdis.nasa.gov";
    final static String modisServerUsername = "MODISDownloader";
    final static String modisServerPassword = "M0D1SD0wnl0ader";

    
    
    public FtpDownload(MADGui madGui, String downloadPath, String saveDir, boolean verbose) {
    	
    	this.madGui = madGui;
        this.ftpFilePath = downloadPath;
        this.localFilePath = saveDir;
        this.verbose = verbose;
        
        
    }    
    
    //This method runs in background. Inherited from SwingWorker.
	@Override
    protected Void doInBackground() throws Exception {
		
		//Creates the connection
		ftpClient = new FTPClient();
		ftpClient.connect(modisFtpServer);
		ftpClient.login(modisServerUsername, modisServerPassword);
		ftpClient.enterLocalPassiveMode();
		ftpClient.setFileType(FTP.BINARY_FILE_TYPE);
        
        try {
 
            byte[] buffer = new byte[4096];//By default 4096
            int bytesRead = -1;
            long totalBytesRead = 0;
            int percentCompleted = 0;
 
            long fileSize=0;
            FTPFile file = null;
            
            try {
            	//Creates a FTPFile , with the info of the file requested to obtain its size
                file = ftpClient.mlistFile(ftpFilePath);
                if (file.equals(null)) {
                    throw new IOException("The path to the file is incorrect");
                }
                //Self explanatory 
                fileSize = file.getSize();
            } catch (IOException ex) {
                throw new IOException("Could not determine the info of the file: "
                        + ex.getMessage());
            }
            
            File fileToDownload = new File(localFilePath);
            FileOutputStream outputStream = new FileOutputStream(fileToDownload);
 
            try {
            	 
                inputStream = ftpClient.retrieveFileStream(ftpFilePath);
                
            } catch (IOException err) {
            	err.printStackTrace();
            }
            //The actual download
            while ((bytesRead = inputStream.read(buffer)) != -1) {
                outputStream.write(buffer, 0, bytesRead);
                totalBytesRead += bytesRead;
                percentCompleted = (int) (totalBytesRead * 100 / fileSize);
                setProgress(percentCompleted);//The Progress Bar notification
            }
 
            outputStream.close();
            inputStream.close();
            
            ftpClient.completePendingCommand();
            
        } catch (IOException err) {
            JOptionPane.showMessageDialog(null,
                    "Error downloading file: " + err.getMessage(), "Connection Error",
                    JOptionPane.ERROR_MESSAGE);
            setProgress(0);
            cancel(true);
            madGui.infoLabel.setText("Try another download directory or check your internet connection");
        } 
        
        //Closes the FTP connection after the download
		if (ftpClient.isConnected()) {
            
			try {
				
				ftpClient.logout();
			
			} catch (IOException e2) {
			
				e2.printStackTrace();
			}
            
            try {
            	
				ftpClient.disconnect();
				
			} catch (IOException e1) {
			
				e1.printStackTrace();
			}
		}
      
        return null;
    }
	
	//Overrides the "Done" methods, that executes a command on the EDT. In this case shows a dialog.
	@Override
    protected void done() {
        if (!isCancelled() && !ftpFilePath.contains(".met") && verbose) {
        	
        	JOptionPane.showMessageDialog(MADGui.frame,"Download Succesful.\nCheck your download folder (Home by default)", "Download Finished",1);
            
        }

    }
	
	
}