/*
 *   The MAD is a simple program that retrieves MODIS and Aster data from the NASA EOSDIS servers.
 *   Copyright (C) 2014  Nicolas Cerna
 *
 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *   
 *   Copyright [2015] [Nicolas Cerna]

	Licensed under the Apache License, Version 2.0 (the "License");
	you may not use this file except in compliance with the License.
	You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

	Unless required by applicable law or agreed to in writing, software
	distributed under the License is distributed on an "AS IS" BASIS,
	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
	See the License for the specific language governing permissions and
	limitations under the License.
 */

/*Future features: 
 * Automatically perform the download at a selected time a day. Scheduled downloads.
 * */

//Gui
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.BufferedReader;

//IO
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
//Lists
import java.util.List;
import java.util.TimeZone;
import java.util.Timer;
//Pattern and Matcher
import java.util.regex.Matcher;
import java.util.regex.Pattern;











//Gui
import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JTabbedPane;
import javax.swing.UIManager;
import javax.swing.UIManager.*;
import javax.swing.UnsupportedLookAndFeelException;












//FTP Library
import org.apache.commons.net.ftp.FTP;
import org.apache.commons.net.ftp.FTPClient;


public class MADGui extends JPanel 
						   implements ActionListener,
						   PropertyChangeListener{
		
		/**
	 * 
	 */
	private static final long serialVersionUID = 1833172050606511424L;
		/**
	 * 
	 */
		
    	static TimeZone GMT = TimeZone.getTimeZone("GMT");
		
    	//Declares a boolean to enable metadata download
		boolean metadata = true;//By default the metadata is also downloaded
	
		//Declares the Content Pane, the Frame and the Panels
		static JFrame frame;
		JComponent modisDownloader; //The "Content Pane"
		JPanel topPanel,botPanel;
		JPanel terraPanel, aquaPanel, downloadButtonPanel, asterPanel, modisCheckPanel;
		JTabbedPane tabbedPane;
		
		//Declares the Download Button
		JButton downloadButton;
		
		//Declares the check box buttons array for the scheduled downloads
		JCheckBox[] modisCheckBoxButtons;
		
		//Declares the Menu and menu items
		JMenuBar menuBar;
		JMenu menuOptions, menuData;
		JMenuItem menuItemChangeDir, menuItemExit, menuItemData;
		JCheckBoxMenuItem menuItemMetadata;
		
		//Declares the Labels
		JLabel infoLabel,statusLabel,titleLabel;
		
		//Declares the progress bar
		JProgressBar progressBar = new JProgressBar(0, 100);
       
        //FTP Data strings
        static String modisStaticFtpFilePath = "/allData/1/";
        static String downloadFtpFilePath = "";
        static String staticLocalFilePath = System.getProperty("user.home")+"/";//Contains the root of the directory where the downloaded files are
        static String localFilePath = staticLocalFilePath;//Contains the root and the name of the file
        String filename ="example.hdf";
           
        //The FTP Client
		FTPClient ftpClient = new FTPClient();
		
		//The Aster url
		String url = "";
		
		//Http Connection
		HttpURLConnection httpConnection;
		HttpURLConnection httpConnection2;
		URL objectUrl;
		
		//Sets the time zone. In this case the GMT.
		
		
		//Misc variables
		long fileSize = 0;
		int connections = 0; 
		
		//String Array with the MODIS TERRA product names
		final static List<String> modisTerraProducts = Arrays.asList("AM1EPHN0","AM1EPHNE","MOD00S","MOD00F","MOD01","MOD03","MOD021KM","MOD02HKM","MOD02QKM","MOD02SSH","MOD07_L2","MOD35_L2",
				"MOD04_L2","MOD05_L2","MOD06_L2","MOD10_L2","MOD10L2C","MOD29","MOD29L2C","MOD14","MOD14CRS","MOD09","MOD09CRS","MOD11_L2","MODMGGAD","MODPTHKM","MODPTQKM",
				"MOD09GA","MOD09GQ","MOD09GHK","MOD09GQK","MOD09GST","MODTBGD","MOD09CMA","MOD09CMG","MOD09A1N","MOD09Q1N","MOD13A4N","MOD13Q4N","MODAODHD");
		
		//String Array with the MODIS AQUA product names
		final static List<String> modisAquaProducts = Arrays.asList("MYDGBAD","MYDGB0","PM1ATTNR_NRT","PM1EPHND_NRT","MYD00S","MYD00F","MYD01","MYD03","MYD021KM","MYD02HKM","MYD02QKM",
				"MYD02SSH","MYD07_L2","MYD35_L2","MYD04_L2","MYD05_L2","MYD06_L2","MYD10_L2","MYD10L2C","MYD29","MYD29L2C","MYD14","MYD14CRS","MYD09","MYD09CRS","MYD11_L2","MYDMGGAD",
				"MYDPTHKM","MYDPTQKM","MYD09GA","MYD09GQ","MYD09GHK","MYD09GQK","MYD09GST","MYDTBGD","MYD09CMA","MYD09CMG","MYDAODHD");
		
		//String Array that may contain any product
		List<String> allModisProducts = new ArrayList<>();
		
		//String Array with the ASTER TERRA product names
		final static List<String> asterProducts = Arrays.asList("AST_L1BE","AST_L1AE","AST_L1B");
		/*Other aster Products not Available:
		"AST_L1BE","AST_L1AE","AST_07","AST_07XT","AST_09","AST_09XT","AST_09T","AST_08","AST_05","AST14OTH","AST_L1B","AST14DMO",
		"AST_L1A","AST14DEM","ASTGTM"*/
				
		//String Array with the TERRA products tooltip texts
		List<String> modisTerraTooltips = Arrays.asList("Spacecraft Ephemeris Data","Extrapolated Orbital Data","L0 PDS Data, Session-Based","L0 PDS Data, 5-Min Swath",
				"L1A Raw Radiances, 5-Min Swath","Geolocation, 5-Min Swath 1km","L1B Calibrated Radiances, 5-Min Swath 1km","L1B Calibrated Radiances, 5-Min Swath 500m",
				"L1B Calibrated Radiances, 5-Min Swath 250m","L1B Subsampled Calibrated Radiances, 5-Min Swath 5km","L2 Temperature and Water Vapor Profiles, 5-Min Swath 5km",
				"L2 Cloud Mask, 5-Min Swath 250m and 1km","L2 Aerosol, 5-Min Swath 10km","L2 Total Precipitable Water Vapor, 5-Min Swath 1km and 5km",
				"L2 Clouds, 5-Min Swath 1km and 5km","L2 Snow Cover, 5-Min Swath 500m","L2 Coarse Snow Cover, 5-Min Swath 5km","L2 Sea Ice Extent, 5-Min Swath 1km",
				"L2 Coarse Sea Ice Extent, 5-Min Swath 5km","L2 Thermal Anomalies / Fire, 5-Min Swath 1km","L2 Coarse Thermal Anomalies / Fire, 5-Min Swath 5km",
				"L2 Surface Reflectance, 5-Min Swath 250m, 500m, and 1km","L2 Coarse Surface Reflectance, 5-Min Swath 5km","L2 Surface Temperature and Emissivity, 5-Min Swath 1km",
				"L2G Daytime Geolocation Angles, Daily 1km","L2G Daytime Pointers, Daily 500m","L2G Daytime Pointers, Daily 250m",
				"L2G Light Surface Reflectance - Daily 500m and 1km","L2G Light Surface Reflectance - Daily 250m","L2G Surface Reflectance - Daily 500m",
				"L2G Surface Reflectance - Daily 250m","L2G Surface Reflectance State - Daily 1km","L2G Daytime Thermal Bands - Daily 1km",
				"L3 Aerosol Optical Thickness, Climate Modeling Grid","L3 Surface Reflectance, Climate Modeling Grid","L3 Daily Rolling 8-Day Surface Reflectance - 500m",
				"L3 Daily Rolling 8-Day Surface Reflectance - 250m","L3 Daily Rolling-8-Day Vegetation Indices - 500m","L3 Daily Rolling-8-Day Vegetation Indices - 250m",
				"L3 Value-added Aerosol Optical Depth");
		
		//String Array with the AQUA products tooltip texts
		List<String> modisAquaTooltips = Arrays.asList("GBAD Data, Session-Based","GBAD Data, 5-Min Swath","GBAD Spacecraft Attitude Data","GBAD Spacecraft Ephemeris Data",
				"L0 PDS Data, Session-Based","L0 PDS Data, 5-Min Swath","L1A Raw Radiances, 5-Min Swath","Geolocation, 5-Min Swath 1km",
				"L1B Calibrated Radiances, 5-Min Swath 1km","L1B Calibrated Radiances, 5-Min Swath 500m","L1B Calibrated Radiances, 5-Min Swath 250m",
				"L1B Subsampled Calibrated Radiances, 5-Min Swath 5km","L2 Temperature and Water Vapor Profiles, 5-Min Swath 5km","L2 Cloud Mask, 5-Min Swath 250m and 1km",
				"L2 Aerosol, 5-Min Swath 10km","L2 Total Precipitable Water Vapor, 5-Min Swath 1km and 5km","L2 Clouds, 5-Min Swath 1km and 5km","L2 Snow Cover, 5-Min Swath 500m",
				"L2 Coarse Snow Cover, 5-Min Swath 5km","L2 Sea Ice Extent, 5-Min Swath 1km","L2 Coarse Sea Ice Extent, 5-Min Swath 5km",
				"L2 Thermal Anomalies / Fire, 5-Min Swath 1km","L2 Coarse Thermal Anomalies / Fire, 5-Min Swath 5km","L2 Surface Reflectance, 5-Min Swath 250m, 500m, and 1km",
				"L2 Coarse Surface Reflectance, 5-Min Swath 5km","L2 Surface Temperature and Emissivity, 5-Min Swath 1km","L2G Daytime Geolocation Angles, Daily 1km",
				"L2G Daytime Pointers, Daily 500m","L2G Daytime Pointers, Daily 250m","L2G Light Surface Reflectance - Daily 500m and 1km","L2G Light Surface Reflectance - Daily 250m",
				"L2G Surface Reflectance - Daily 500m","L2G Surface Reflectance - Daily 250m","L2G Surface Reflectance State - Daily 1km","L2G Daytime Thermal Bands - Daily 1km",
				"L3 Aerosol Optical Thickness, Climate Modeling Grid","L3 Surface Reflectance, Climate Modeling Grid","L3 Value-added Aerosol Optical Depth");
		
		List<String> asterTooltips = Arrays.asList("Registered Radiance at the Sensor - Expedited","Reconstructed Unprocessed Instrument Data - Expedited",
				"Surface Reflectance - VNIR & SWIR","Surface Reflectance - VNIR & Crosstalk Corrected SWIR","Surface Radiance - VNIR & SWIR",
				"Surface Radiance - VNIR & Crosstalk Corrected SWIR","Surface Radiance TIR","Surface Kinetic Temperature","Surface Emissivity",
				"Registered Radiance at the Sensor - Orthorectified","Registered Radiance at the Sensor",
				"Digital Elevation Model & Registered Radiance at the Sensor - Orthorectified","Reconstructed Unprocessed Instrument Data","Digital Elevation Model",
				"ASTER Global Digital Elevation Model");
		
		//String that contains the download modes
		String[] opt = {"ALL_FILES","METADATA_ONLY","HDF_ONLY"};
		String[] opt2 = {"ALL_PAST_FILES","ONLY_FUTURE_FILES"};
		
		//Timer list for scheduled downloads
		Timer[] timerList;
		
		
		/*CONSTRUCTOR*/
	    MADGui() {
	        super(new BorderLayout());//Call the Superclass constructor
	        /////////////////////////////////////////////////////////////////////////////////
			//The button groups
			ButtonGroup modisTerraButtonGroup = new ButtonGroup();
	        ButtonGroup modisAquaButtonGroup = new ButtonGroup();
	        ButtonGroup asterButtonGroup = new ButtonGroup();
	        
	      //Initilizes the radio buttons and the download buttons
	        terraPanel =  new JPanel(new GridLayout(8, 5));
	        terraPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createMatteBorder(2,2,2,2,Color.decode("#c77f25")), "Terra Products"));
	        terraPanel.setBackground(Color.decode("#dedede"));
	        
	        aquaPanel =  new JPanel(new GridLayout(8, 5));		
	        aquaPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createMatteBorder(2,2,2,2,Color.decode("#789ea3")), "Aqua Products"));
	        aquaPanel.setBackground(Color.decode("#dedede"));
	        
	        asterPanel =  new JPanel(new GridLayout(8, 5));		
	        asterPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createMatteBorder(2,2,2,2,Color.decode("#c77f25")), "Terra Products"));
	        asterPanel.setBackground(Color.decode("#dedede"));
	        
	        modisCheckPanel =  new JPanel(new GridLayout(8, 10));		
	        modisCheckPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createMatteBorder(2,2,2,2,Color.decode("#4387dc")), "Modis Terra and Aqua Products"));
	        modisCheckPanel.setBackground(Color.decode("#dedede"));
	        
	        
	        //Initializes the radio buttons.
	        JButton[] terraProductsButtons = new JButton[modisTerraProducts.size()];
	        for(int i = 0; i< modisTerraProducts.size(); i++){
	        	terraProductsButtons[i] = new JButton(modisTerraProducts.get(i));
	        	terraProductsButtons[i].setActionCommand(modisTerraProducts.get(i));
	        	terraProductsButtons[i].setToolTipText(modisTerraTooltips.get(i));
	        	terraProductsButtons[i].addActionListener(this);
	        	
	        	modisTerraButtonGroup.add(terraProductsButtons[i]);
	        	
	        	terraPanel.add(terraProductsButtons[i]);
	        	
	        }
	        
	        JButton[] aquaProductsButtons = new JButton[modisAquaProducts.size()];
	        for(int i = 0; i< modisAquaProducts.size(); i++){
	        	aquaProductsButtons[i] = new JButton(modisAquaProducts.get(i));
	        	aquaProductsButtons[i].setActionCommand(modisAquaProducts.get(i));
	        	aquaProductsButtons[i].setToolTipText(modisAquaTooltips.get(i));
	        	aquaProductsButtons[i].addActionListener(this);
	        	
	        	modisAquaButtonGroup.add(aquaProductsButtons[i]);
	        	
	        	aquaPanel.add(aquaProductsButtons[i]);
	        }
	        
	        JButton[] asterProductsButtons = new JButton[asterProducts.size()];
	        for(int i = 0; i< asterProducts.size(); i++){
	        	asterProductsButtons[i] = new JButton(asterProducts.get(i));
	        	asterProductsButtons[i].setActionCommand(asterProducts.get(i));
	        	asterProductsButtons[i].setToolTipText(asterTooltips.get(i));
	            asterProductsButtons[i].addActionListener(this);
	            
	            asterButtonGroup.add(asterProductsButtons[i]);
	            
	            asterPanel.add(asterProductsButtons[i]);
	        }
	        ///////////////////////////////////////////////////////////////////////////
	        //Initializes the check boxes
	        allModisProducts.addAll(modisTerraProducts);
    		allModisProducts.addAll(modisAquaProducts);
	        
	        modisCheckBoxButtons = new JCheckBox[allModisProducts.size()];
	        
	        for(int i = 0; i< allModisProducts.size(); i++){
	        	modisCheckBoxButtons[i] = new JCheckBox(allModisProducts.get(i));
	        	modisCheckBoxButtons[i].setActionCommand(allModisProducts.get(i));
	        	modisCheckBoxButtons[i].addActionListener(this);
	        	
	        	if(i < modisTerraProducts.size()){
	        		modisCheckBoxButtons[i].setToolTipText(modisTerraTooltips.get(i));
	        	}
	        	else{
	        		modisCheckBoxButtons[i].setToolTipText(modisAquaTooltips.get(i-modisTerraProducts.size()));
	        	}
	        	
	        	modisCheckPanel.add(modisCheckBoxButtons[i]);
	        }
	        /////////////////////////////////////////////////////////////////////////////
	        //Initializes the Download Button
	        downloadButton = new JButton("Download");
	        downloadButton.setMnemonic(KeyEvent.VK_D);
	        downloadButton.addActionListener(this);
	        downloadButton.setActionCommand("Download");
	        downloadButton.setToolTipText("Download the current file");
	        
	        //Initializes the menu bar
	        menuBar = new JMenuBar();
	        
	        //Initializes the menus
			menuOptions = new JMenu("Options");
			menuOptions.setMnemonic(KeyEvent.VK_O);
			menuOptions.setToolTipText("Display the options");
			
			menuData = new JMenu("Database");
			menuData.setMnemonic(KeyEvent.VK_O);
			menuData.setToolTipText("Database Actions");
			
			//Initializes the menu items
			menuItemChangeDir = new JMenuItem("Change Donwload Directory");
			menuItemChangeDir.addActionListener(this);
			menuItemChangeDir.setActionCommand("ChangeDir");
			
			menuItemMetadata = new JCheckBoxMenuItem("Download Metadata");
			menuItemMetadata.addActionListener(this);
			menuItemMetadata.setActionCommand("Metadata");
	        
			menuItemExit = new JMenuItem("Exit");
			menuItemExit.addActionListener(this);
			menuItemExit.setActionCommand("Exit");
			
			menuItemData = new JMenuItem("Update Database");
			menuItemData.addActionListener(this);
			menuItemData.setActionCommand("Update Database");
			
	        //Initializes the progress bar
	        progressBar = new JProgressBar(0,100);
	        progressBar.setPreferredSize(new Dimension(200, 30));
	        progressBar.setStringPainted(true);
	        
	        //Initializes the Labels
	        infoLabel = new JLabel("Select a product to begin",JLabel.CENTER);
	        infoLabel.setToolTipText("Displays the instructions and more");
	        infoLabel.setFont(new Font("calibri", Font.PLAIN, 12));
	        infoLabel.setBorder(BorderFactory.createEmptyBorder(20,20,0,20));
	        infoLabel.setForeground(Color.decode("#333333"));
	        
	        statusLabel = new JLabel("Disconnected");
	        statusLabel.setToolTipText("Displays the status of the program");
	        statusLabel.setFont(new Font("calibri", Font.BOLD, 20));
	        statusLabel.setBorder(BorderFactory.createEmptyBorder(5,0,0,175));
	        statusLabel.setForeground(Color.LIGHT_GRAY);
	        
	        titleLabel = new JLabel("MAD - Modis and Aster Downloader");
	        titleLabel.setFont(new Font("calibri", Font.PLAIN, 20));
	        titleLabel.setBorder(BorderFactory.createEmptyBorder(0,120,0,0));
	        titleLabel.setForeground(Color.decode("#6d6d6d"));
	        //////////////////////////////////////////////////////////////////////////////////
	        //PANELS
	        
	        //Creates the tabbed panel
	        tabbedPane = new JTabbedPane();
	        add(tabbedPane);//Adds the tab to the main window 
	        
	        //Creates the tabs
	        JPanel tab1 = new JPanel(new BorderLayout());
	        JPanel tab2 = new JPanel(new BorderLayout());
	        JPanel tab3 = new JPanel(new BorderLayout());
	        
	        //Creates the download panel
	        downloadButtonPanel = new JPanel(new BorderLayout());
	        downloadButtonPanel.setBorder(BorderFactory.createEmptyBorder(20,480,20,480));
	        
	        //Puts the radiopanels on the tabs
	        tab1.add(terraPanel, BorderLayout.LINE_START);
	        tab1.add(aquaPanel, BorderLayout.CENTER);  
	        tab2.add(asterPanel, BorderLayout.CENTER);
	        tab3.add(modisCheckPanel);
	        
	        //Placing the Menu and adding its items
	        menuOptions.add(menuItemChangeDir);
	        menuOptions.add(menuItemMetadata);
	        menuOptions.add(menuItemExit);
	        
	        menuData.add(menuItemData);
	        
	        menuBar.add(menuOptions);
	        menuBar.add(menuData);
	        
	        menuItemMetadata.setSelected(true);//Metadata its downloaded by default along the hdf file
	        
	        //Placing the Labels
	        topPanel = new JPanel(new BorderLayout());
	        topPanel.add(titleLabel, BorderLayout.LINE_START);
	        topPanel.add(statusLabel, BorderLayout.LINE_END);
	        topPanel.add(menuBar, BorderLayout.PAGE_START);
	        topPanel.setBackground(Color.decode("#dedede"));
	        topPanel.setBorder(BorderFactory.createEmptyBorder(0,0,0,0));
	        add(topPanel, BorderLayout.PAGE_START);//It goes in the upper Container
	        
	        //Placing the Information Label and the Progress Bar
	        botPanel = new JPanel(new BorderLayout());
	        botPanel.add(progressBar, BorderLayout.CENTER);
	        botPanel.add(infoLabel, BorderLayout.PAGE_END);
	        botPanel.setBackground(Color.decode("#dedede"));
	        botPanel.setBorder(BorderFactory.createEmptyBorder(10,20,10,20));
	        add(botPanel, BorderLayout.PAGE_END);
	        
	        //Puts the download button on the bottom panel
	        downloadButtonPanel.add(downloadButton, BorderLayout.CENTER);
	        
	        //Puts the button panel on the tab1
	        botPanel.add(downloadButtonPanel, BorderLayout.PAGE_START);
	        
	        //The Download Button is not enabled by default
	        downloadButton.setEnabled(false);
	        
	        //Finally adds the tabs to the tabbed pane
	        tabbedPane.addTab("Modis Products", null,tab1, "Terra & Aqua MODIS Products");
	        tabbedPane.addTab("Aster Products", null, tab2, "Terra ASTER Products");
	        tabbedPane.addTab("Scheduled Downloads", null, tab3, "Scheduled Database Updates");
	        
	        //Sets the border of the main window
	        setBorder(BorderFactory.createEmptyBorder(10,10,10,10));
	        
	        //Creates timers for the scheduled Downloads
	        timerList = new Timer[78];//There are 78 Modis products 
	    }//End of Constructor
	    ///////////////////////////////////////////////////////////////////////////////////////////////////////////////
	    /*LISTENERS*/
	    public void actionPerformed(ActionEvent e) {
	    	//When you select a MODIS product://////////////////////////////////////////////////////////////////////////
	    	if (allModisProducts.contains(e.getActionCommand()) && tabbedPane.getSelectedIndex() == 0){
	    		//First, we try to establish a new connection (Yes, this is performed every time!)
	    		try{ 
		    		//Creates the connection
	    			ftpClient.connect(FtpDownload.modisFtpServer);
	    			boolean loginSuccess = ftpClient.login(FtpDownload.modisServerUsername, FtpDownload.modisServerPassword);
	    			ftpClient.enterLocalPassiveMode();
	    			ftpClient.setFileType(FTP.BINARY_FILE_TYPE);
	    			
	    			
	    			if(loginSuccess){
	    				infoLabel.setForeground(Color.decode("#333333"));
	    				infoLabel.setText("Login Successful");
	    				statusLabel.setText("Connected");
	    				statusLabel.setForeground(Color.decode("#617889"));
	    			}
 	           
	    		}catch (IOException err) {
	
    	            infoLabel.setText("Connection Error. Try again later.\n"+err.getMessage());
    	            err.printStackTrace();
    	        }
	    		
	    		//Adds the product to the file path 
	    		downloadFtpFilePath = modisStaticFtpFilePath.concat(e.getActionCommand());
	    		
	    		//Resets the infoLabel and the Progress Bar
	    		infoLabel.setForeground(Color.decode("#333333"));
	    		infoLabel.setText("");
	    		progressBar.setValue(0);
	    		
	    		//Return the download button to "enabled" after the options are selected
	    		if(ftpClient.isConnected()){
	    			
	    			downloadButton.setEnabled(true);
	
	    		}
	    		//Opens a dialog to specify the day
	    		if(ftpClient.isConnected()){
	    			try {
	    				//Stores the present year
	    				String year = MADGui.getYear();
	    				
	    				
	    				//Stores the available days for a product the present year
	    				String[] daysAvailable = ftpClient.listNames("allData/1/".concat(e.getActionCommand()).concat("/"+year+"/"));
	    				
	    				String[] filesAvailable = null;
	    				String daySelected = daysAvailable[0];
	    				
	    				//Asks for the day you want to access the data, showing only the available options
	    				daySelected = (String)JOptionPane.showInputDialog(frame, "Select a day of the year.\n"
	    								+ "Only available days are shown:",
	    								"Day Selector", JOptionPane.QUESTION_MESSAGE, null, daysAvailable, daysAvailable[0]);
	    					
	    				//Proceeds only if the user makes a choice (not null)
	    				if(!String.valueOf(daySelected).matches("null")){
	    					//Adds the day to file path to future download
		    				downloadFtpFilePath = downloadFtpFilePath.concat("/"+year+"/"+daySelected+"/");   				   
		    				setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
		    				infoLabel.setText("Loading... Please wait");
		    				botPanel.repaint();
		    				
		    				filesAvailable = ftpClient.listNames(downloadFtpFilePath);
		    						
	    				}
	    					
	    				//Prints the available files, only if there are available files (Not null and non zero length array)
	    				if(!String.valueOf(filesAvailable).matches("null")){
	    					//Ask for the file you want to access the data, showing only the available options
		    				if(filesAvailable.length > 0){
		    					filename = (String)JOptionPane.showInputDialog(frame, "Select a file.\n"
		    								+ "Only available files are shown:",
		    								"File Selector", JOptionPane.QUESTION_MESSAGE, null,filesAvailable, filesAvailable[0]);
		    					//Updates the path with the selected filename
		    					if(!String.valueOf(filename).matches("null")){
		    						localFilePath = staticLocalFilePath;
		    						localFilePath = localFilePath.concat(filename);
		    						downloadFtpFilePath = downloadFtpFilePath.concat(filename);
		    						setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
		    						infoLabel.setText("Now press the DOWNLOAD button to complete your request");
		    					}
		    					else{
		    						infoLabel.setText("You must select a day and a product to proceed");
		    						setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
		    					}
		    	    		}
	    				}
	    				//Prints that there are no available files (Not null, but it's a zero length array)
	    				if(!String.valueOf(filesAvailable).matches("null")){
	    					if(filesAvailable.length == 0){
	    						infoLabel.setText("There are no available files for the day you selected");
	    						setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
	    						downloadButton.setEnabled(false);
	    					}
	    				}
	    				//Prints that there are no available files (Its null. This happens sometimes when you don't select a day after a download)
	    				if(String.valueOf(filesAvailable).matches("null")){
	    					infoLabel.setText("You must select a day to proceed the download.");
    						setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
    						downloadButton.setEnabled(false);
	    				}
	    					
	    			} catch (IOException err) {
	    				//If its the first time we attempt a connection to the backup server
	    				if(connections == 0){
	    					err.printStackTrace();
	    					infoLabel.setForeground(Color.RED);
	    					infoLabel.setText("CONECTION ERROR: Main server may be unnavailable.\n Trying to connect to Backup Server...");
	    				
	    					//If the main server is down, then we try to connect to the backup server
	    					try {
	    					
	    						ftpClient.connect(FtpDownload.backupftpserver);
	    						boolean loginSuccess = ftpClient.login(FtpDownload.modisServerUsername, FtpDownload.modisServerPassword);
	    						connections++;
	    						
	    						if(loginSuccess){
	    							
	    							infoLabel.setText("Now you are connected to the backup server.\nIf the connection doesn't work properly try again later.");
	    						}
							
	    					} catch (IOException err1) {
						
	    						err1.printStackTrace();
	    					}
	    				}
	    				
	    				//If the connection to the backup server fails, then we logout and disconnect.
	    				else{
	    					
	    					if (ftpClient.isConnected()) {
	    	                    
	    						try {
	    							
									ftpClient.logout();
								
	    						} catch (IOException e2) {
								
									e2.printStackTrace();
								}
	    	                    
	    	                    try {
	    	                    	
									ftpClient.disconnect();
									
								} catch (IOException e1) {
								
									e1.printStackTrace();
								}
	    	                    
	    	                    infoLabel.setForeground(Color.decode("#333333"));
	    	                    infoLabel.setText("Now you are disconnected from the server. The Main and Backup servers do not respond. Try again later.");
	    	                    statusLabel.setText("DISCONNECTED");
	    	                    statusLabel.setForeground(Color.LIGHT_GRAY);
	    	                    downloadButton.setEnabled(false);
	    	                    progressBar.setValue(0);
	    	                    connections = 0; //Numbers of connections reset
	    	                 }
	    					
	    				}
	    				
	    			}
	    		}
	    		
	    		//When you select a product without a connection.
	    		else{
	    			
	    			infoLabel.setText("Please check your internet connection and try a to select product again");
	    		}
	    	}//end if (Modis Products)
	    	//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	    	//When you select an ASTER product:
	    	else if (asterProducts.contains(e.getActionCommand())){
	    		
	    		StringBuffer response = new StringBuffer();
	    		List<String> daysList = new ArrayList<String>();
	    		
	    		try{
	    		//First we try to establish a connection
	    		url = UrlDownload.staticAsterUrl.concat(e.getActionCommand() + ".003");
	    		objectUrl = new URL(url);
	    		httpConnection = (HttpURLConnection) objectUrl.openConnection();
	     
	    		// optional default is GET
	    		httpConnection.setRequestMethod("GET");
	     
	    		//add request header
	    		httpConnection.setRequestProperty("User-Agent", "Mozilla/5.0");
	     
	    		String responseCode = String.valueOf(httpConnection.getResponseCode());
	    		infoLabel.setText(responseCode);
	    		statusLabel.setText("Connected");
				statusLabel.setForeground(Color.decode("#617889"));
	    			
	    		}catch (Exception err) {	
	    			err.printStackTrace();
	    			infoLabel.setText("Connection Error");
	    			
	    		}
	    		
	    		//Resets the infoLabel and the Progress Bar
	    		infoLabel.setForeground(Color.decode("#333333"));
	    		infoLabel.setText("");
	    		progressBar.setValue(0);
	    		
	    		//Return the download button to "enabled" after the options are selected
	    		downloadButton.setEnabled(true);
	    		
	    		//Opens a dialog to specify the day
	    				    				
	    		//Stores the available days for a product
	    		try{
	    		BufferedReader in = new BufferedReader(
	    		        new InputStreamReader(httpConnection.getInputStream()));
	    		String inputLine;
	     
	    		while ((inputLine = in.readLine()) != null) {
	    			response.append(inputLine);
	    		}
	    		in.close();
	    		
	    		}catch(Exception err){
	    			err.printStackTrace();
	    		}
	    		
	    		Pattern p = Pattern.compile("href=(.*?)>");
	    		Matcher m = p.matcher(response.toString());
	    		
	    		int counter = 0; //Useful to eliminate some error matches
	    		while(m.find()){
	    			if(counter > 4){	
	    				daysList.add(m.group().substring(6, 16).replace('.', '-'));
	    			}
	    				counter++;
	    		}
	    		
	    		String[] daysAvailable = daysList.toArray(new String[daysList.size()]);
	    				
	    		String[] filesAvailable = null;
	    		String daySelected = daysAvailable[0];
	    				
	    		
	    		//Asks for the day you want to access the data, showing only the available options
	    		daySelected = (String)JOptionPane.showInputDialog(frame, "Select a day.\n"
	    						+ "Only available days are shown:",
	    						"Day Selector", JOptionPane.QUESTION_MESSAGE, null, daysAvailable, daysAvailable[0]);
	    				
	    		//Proceeds only if the user makes a choice (not null)
	    		if(!String.valueOf(daySelected).matches("null")){
	    			//Adds the day to file path to future download
		    		url = url.concat("/"+daySelected.replace('-','.')+"/");   				   
		    		setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
		    		infoLabel.setText("Loading... Please wait");
		    		botPanel.repaint();
		    		
		    		response = new StringBuffer();
		    		List<String> fileList = new ArrayList<String>();
		    		try{
			    		//First we try to establish a connection to the new url
			    		URL objectUrl = new URL(url);
			    		httpConnection = (HttpURLConnection) objectUrl.openConnection();
			     
			    		//Optional default is GET
			    		httpConnection.setRequestMethod("GET");
			     
			    		//Add request header
			    		httpConnection.setRequestProperty("User-Agent", "Mozilla/5.0");
			     
			    		String responseCode = String.valueOf(httpConnection.getResponseCode());
			    		infoLabel.setText(responseCode);
			    			
			    		}catch (Exception err) {	
			    			err.printStackTrace();
			    			infoLabel.setText("Connection Error");
			    		}
		    		
		    		try{//This download the http response an reads it to get the days available and the products
			    		BufferedReader in = new BufferedReader(
			    		        new InputStreamReader(httpConnection.getInputStream()));
			    		String inputLine;
			     
			    		while ((inputLine = in.readLine()) != null) {
			    			response.append(inputLine);
			    		}
			    		in.close();
			    		
			    		}catch(Exception err){
			    			err.printStackTrace();
			    		}
			    		
			    		m = p.matcher(response.toString());
			    		
			    		counter = 0; //Useful to eliminate some error matches
			    		while(m.find()){
			    			if(counter > 4){	
			    				fileList.add(m.group().substring(6,m.group().length()-2));
			    			}
			    				counter++;
			    		}
			    		
			    		filesAvailable = fileList.toArray(new String[fileList.size()]);
		   					
	    		}
	    				
	    		//Prints the available files, only if there are available files (Not null and non zero length array)
	    		if(!String.valueOf(filesAvailable).matches("null")){
	    			//Ask for the file you want to access the data, showing only the available options
		    		if(filesAvailable.length > 0){
		    			filename = (String)JOptionPane.showInputDialog(frame, "Select a file.\n"
		    						+ "Only available files are shown:",
		    						"File Selector", JOptionPane.QUESTION_MESSAGE, null,filesAvailable, filesAvailable[0]);
		    			//Updates the path with the selected filename
		    			if(!String.valueOf(filename).matches("null")){
		    				localFilePath = staticLocalFilePath;
		    				localFilePath = localFilePath.concat(filename);
		    				url = url.concat(filename);
		    				try {
		    					//Creates the URL object with the desired url
								objectUrl = new URL(url);
							} catch (MalformedURLException err) {
								err.printStackTrace();
							}
		    				setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
		    				infoLabel.setText("Now press the DOWNLOAD button to complete your request");
		    			}
		    			
		    			else{
		    				infoLabel.setText("You must select a day and a product to proceed");
		    				setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
		    			}
		    		}
	    		}
	    				//Prints that there are no available files (Not null, but it's a zero length array)
	    				if(!String.valueOf(filesAvailable).matches("null")){
	    					if(filesAvailable.length == 0){
	    						infoLabel.setText("There are no available files for the day you selected");
	    						setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
	    						downloadButton.setEnabled(false);
	    					}
	    				}
	    				//Prints that there are no available files (Its null. This happens sometimes when you don't select a day after a download)
	    				if(String.valueOf(filesAvailable).matches("null")){
	    					infoLabel.setText("You must select a day to proceed the download.");
    						setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
    						downloadButton.setEnabled(false);
	    				}
	    	}//End else (Aster products)
	    	/////////////////////////////////////////////////////////////////////////////////////////////////
	    	//When you select a Modis product for a Scheduled Download
	    	else if(allModisProducts.contains(e.getActionCommand()) && tabbedPane.getSelectedIndex() == 2 ){
	    		try{ 
	    			//Creates the connection
	    			ftpClient.connect(FtpDownload.modisFtpServer);
	    			ftpClient.login(FtpDownload.modisServerUsername, FtpDownload.modisServerPassword);
	    			ftpClient.enterLocalPassiveMode();
	    			ftpClient.setFileType(FTP.BINARY_FILE_TYPE);
	    			
	    			}catch (IOException err) {
	    				err.printStackTrace();
	    			}
	    		
	    		//Index of the modis products in the array
	    		int index= allModisProducts.indexOf(e.getActionCommand());
	    		String mode = null, option = null;
	    		
	    		//Only if the button is selected
	    		if(modisCheckBoxButtons[index].isSelected()){
	    		
	    		mode = (String) JOptionPane.showInputDialog(frame, "Select a mode for the download schudule:\n"
						+ "All Files (HDF + .met) \nMetadata Only \nHDF Files Only",
						"Mode Selector", JOptionPane.QUESTION_MESSAGE, null, opt, opt[0]);
	    		}

	    		
	    		//If the mode is properly selected and the check box button is selected
	    		if(mode != null && modisCheckBoxButtons[index].isSelected()){
	    			//Mode selection: Every single file available, or the files available from now on
	    			option = (String) JOptionPane.showInputDialog(frame, "Select a mode for the download schudule:\n"
							+ "All Available Files on the Server \nOnly new files available from now on",
							"Option Selector", JOptionPane.QUESTION_MESSAGE, null, opt2, opt2[0]);
	    			
	    			//String necessaries to the next part
	    			String lastDay = null, lastFile = null;
	    			//If the mode is the one that is only for new files
	    			if(option == "ONLY_FUTURE_FILES" ){
	    				//Stores the available days for a product the present year
	    				String[] daysAvailable =null;
	    				String[] lastFiles = null;
	    				
						
	    				try {
							daysAvailable = ftpClient.listNames("allData/1/".concat(e.getActionCommand()).concat("/"+MADGui.getYear()+"/"));
						} catch (IOException e1) {
				
							e1.printStackTrace();
						}
	    				
						lastDay = daysAvailable[daysAvailable.length-1];
	    				
						try {
							lastFiles = ftpClient.listNames("allData/1/".concat(e.getActionCommand()).concat("/"+MADGui.getYear()+"/"+daysAvailable[daysAvailable.length-1]));
						} catch (IOException e1) {
							
							e1.printStackTrace();
						}
	    				
						lastFile = lastFiles[lastFiles.length-1];
	    				
	    			}
	    			
	    			if(option != null){
	    				
	    				timerList[index] = new Timer();
	    				timerList[index].schedule(new ScheduledDownload(e.getActionCommand(), mode, option,lastDay,lastFile,MADGui.getYear()), 10, 5*60000);//Starts the task every 5 minutes
	    				infoLabel.setText("Task for " +e.getActionCommand()+ " scheduled successfully");
	    			}
	            }
	    		
	    		//If the button is deselected, the we cancel the timer!
	    		if(!modisCheckBoxButtons[index].isSelected()){
	    			
	    			timerList[index].cancel();
	    			infoLabel.setText("Task for " +e.getActionCommand()+ " cancelled successfully");
	    			
	    		}
	 
	    	}
	    	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	    	//When you change the download directory
	    	else if(e.getActionCommand().matches("ChangeDir")){
	    		
	    		JFileChooser chooser = new JFileChooser();
	    	    chooser.setCurrentDirectory(new java.io.File(System.getProperty("user.home")));
	    	    chooser.setDialogTitle("Choose a Directory");
	    	    chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
	    	    chooser.setAcceptAllFileFilterUsed(false);
	    	    String dir = null; 
	    	    
	    	    if (chooser.showOpenDialog(null) == JFileChooser.APPROVE_OPTION) {
	    	      dir = chooser.getSelectedFile().toString();
	    	    } 

	    		if( dir != null){
	    			if(!dir.isEmpty()){
	    				staticLocalFilePath = dir+ "/";
	    			}
	    			else{
	    				staticLocalFilePath = System.getProperty("user.home")+"/"; 
	    			}
	    		}
	    		else{
	    			staticLocalFilePath = System.getProperty("user.home")+"/";
	    		}
	    	}
	    	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	    	//When you select "Enable/Disable Metadata"
	    	else if(e.getActionCommand().matches("Metadata")){
	    		int option = JOptionPane.showConfirmDialog(frame, "Download metadata if available?", "Enable/Disable metadata download", 0);
	    		
	    		if(option == 0){
	    			metadata = true;
	    			menuItemMetadata.setSelected(true);
	    		}
	    		else{
	    			metadata = false;
	    			menuItemMetadata.setSelected(false);
	    		}
	    	}
	    	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	    	//When you select "Exit"
	    	else if(e.getActionCommand().matches("Exit")){
	    		int option = JOptionPane.showConfirmDialog(frame, "Do you really want to exit?", "Exit", 0);
	    		
	    		if(option == 0){
	    			System.exit(0);
	    		}
	    		else{
	    			/*Does nothing, just for order*/
	    		}
	    	}
	    	////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	    	//When you select "Update database"
	    	else if(e.getActionCommand().matches("Update Database")){
	    		String mode = null, option =null;
	    		int confirm = JOptionPane.showConfirmDialog(frame, "Do you really want to update?.\nThis could take some time.", "Update Database Confirmation", 0);
	    		
	    		if(confirm == 0){
	    			String product = (String)JOptionPane.showInputDialog(frame, "Select a product.\n"
    						+ "Only available products are shown:",
    						"File Selector", JOptionPane.QUESTION_MESSAGE, null,allModisProducts.toArray(), allModisProducts.toArray()[0]);
	    			//Checks for answering "cancel" in the option pop up window
	    			mode= null;
	    			
	    			if(product != null){
	    				
	    				mode = (String) JOptionPane.showInputDialog(frame, "Select a mode for the update:\n"
	    						+ "All Files (HDF + .met) \nMetadata Only \nHDF Files Only",
	    						"Mode Selector", JOptionPane.QUESTION_MESSAGE, null, opt, opt[0]);
	    			}
	    			
	    			if(mode != null){
	    				option = (String) JOptionPane.showInputDialog(frame, "How do you like to perform the updates?:\n\n"
	    						+ "Download all past uploaded files available on the server \n\nOnly download files uploaded to the server from now on",
	    						"Mode Selector", JOptionPane.QUESTION_MESSAGE, null, opt2, opt2[0]);
	    				
	    				if(option != null){
	    					new DatabaseDownload(product, mode, option, "-1", "-1","-1").execute();
	    				}
	    			}
	    		}
	    		else{
	    			/*Does nothing, just for order*/
	    		}
	    	}
	    	/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	    	//When you press the Download Button:
	    	else if(e.getActionCommand().matches("Download")){
	    			//When is a Modis Download////////////////////////////////////////////////////////////////////
	    			if(tabbedPane.getSelectedIndex() == 0){
	    		
	    				progressBar.setValue(0);
	    				FtpDownload downloadHdf = new FtpDownload(this, downloadFtpFilePath, localFilePath,true);
	    				downloadHdf.addPropertyChangeListener(this); // For monitoring the percentage completition
				
	    				infoLabel.setForeground(Color.decode("#333333"));
	    				infoLabel.setText("Downloading... Please Wait");
	    				repaint();
	    				//When the download is taking place the download button is disabled
	    				downloadButton.setEnabled(false);	
	    				downloadHdf.execute();    	
	    				
	    				//Downloads the metadata
	    				//Checks if the selected file is already a metadata with the "contains" method
	    				//It also checks that the metadata exists before downloading it
	    				if(metadata && !downloadFtpFilePath.contains(".met")){
	    					boolean checkFile = false;
	    	    			try {
	    	    				//Metadata existence check
								String[] checkArray = ftpClient.listNames(downloadFtpFilePath.substring(0, downloadFtpFilePath.lastIndexOf("/")));
								for(int i =0; i < checkArray.length; i++){
									if(checkArray[i].equals(downloadFtpFilePath.substring(downloadFtpFilePath.lastIndexOf("/")+1) +".met")){
										checkFile = true;
									}
								}
								
							} catch (IOException e1) {
								
								e1.printStackTrace();
							}
	    	    			
	    	    			if(checkFile){

	    	    				FtpDownload downloadMetadata = new FtpDownload(null, downloadFtpFilePath + ".met", localFilePath + ".met",true);
	    	    				downloadButton.setEnabled(false);	
	    	    				downloadMetadata.execute();
		    				}
	    					
	    				}
	    				
	    			}
	    			//When is an Aster Download///////////////////////////////////////////////////////////////////////////////////
	    			else{
	    				UrlDownload downloadHdf = new UrlDownload(this, objectUrl, localFilePath, true);
	    				downloadHdf.addPropertyChangeListener(this); // For monitoring the percentage completion
	    				
	    				infoLabel.setForeground(Color.decode("#333333"));
	    				infoLabel.setText("Downloading... Please wait");
	    				repaint();
	    				//When the download is taking place the download button is disabled
	    				downloadButton.setEnabled(false);	
	    				downloadHdf.execute();
	    				
	    				//Downloads the metadata
	    				//Check if the selected file is not already a metadata
	    				if(metadata && !url.contains(".xml")){
	    					//Updates the url to fit the xml extension
	    		    		try {
								objectUrl = new URL(url + ".xml");
							} catch (MalformedURLException e1) {
								e1.printStackTrace();
							}
	    		    		//Open a new http connection
	    		    		try {
								httpConnection2 = (HttpURLConnection) objectUrl.openConnection();
							} catch (IOException e1) {
								e1.printStackTrace();
							}
	    		    		
	    					UrlDownload downloadMetadata = new UrlDownload(null, objectUrl, localFilePath + ".xml", true);
		    				
		    				//When the download is taking place the download button is disabled
		    				downloadButton.setEnabled(false);	
		    				downloadMetadata.execute();
	    					
	    				}
	    			}
		    }
	    	
	    	else{/*Default Action: Nothing*/}
	    	
	    }//End of Action Listeners
	    /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    
	    /**
	     * Create the GUI and show it.  For thread safety,
	     * this method should be invoked from the
	     * event-dispatching thread.
	     */
	    static void createAndShowGUI() {
	    	try {
	    		//Sets up the Look and Feel of the program. "Nimbus" by default, if not "Metal".
	    		try {
	    		    for (LookAndFeelInfo info : UIManager.getInstalledLookAndFeels()) {
	    		        if ("Nimbus".equals(info.getName())) {
	    		            UIManager.setLookAndFeel(info.getClassName());
	    		            break;
	    		        }
	    		    }
	    		} catch (Exception e) {
	    			UIManager.setLookAndFeel(UIManager.getCrossPlatformLookAndFeelClassName());
	    		} 
				/*You can change the L&F with setLookAndFeel even after the program's GUI is visible. To make existing components reflect the new L&F, invoke the
				SwingUtilities updateComponentTreeUI method once per top-level container. Then you might wish to resize each top-level container to reflect the
				new sizes of its contained components. For example:
				UIManager.setLookAndFeel(lnfName);
				SwingUtilities.updateComponentTreeUI(frame);
				frame.pack();*/
			} catch (ClassNotFoundException err) {
				
				err.printStackTrace();
			} catch (InstantiationException err) {
				
				err.printStackTrace();
			} catch (IllegalAccessException err) {
				
				err.printStackTrace();
			} catch (UnsupportedLookAndFeelException err) {
				
				err.printStackTrace();
			}
	    	
	        //Creates and set up the window.
	        frame = new JFrame("MAD Downloader");
	        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	 
	        //Creates and set up the content pane.
	        JComponent modisDownloader = new MADGui();
	        modisDownloader.setOpaque(true); //Content panes must be opaque
	        modisDownloader.setBackground(Color.decode("#dedede"));
	        frame.setContentPane(modisDownloader);
	 
	        //Displays the window.
	        frame.pack();
	        frame.setResizable(false);
	        frame.setVisible(true);
	    }
	    
	    //Download Success Setter
	    void setFileSize(long size){
	    	fileSize = size;
	    }
	    
	    //The progress bar "Event Listener"
	    @Override
		public void propertyChange(PropertyChangeEvent evt) {
	    	if ("progress" == evt.getPropertyName()) {
	            int progress = (Integer) evt.getNewValue();
	            progressBar.setValue(progress);
	        }
			
		}
	    
		//Returns a string that contains the present year
		static String getYear(){
			return String.valueOf(Calendar.getInstance(GMT).get(Calendar.YEAR));
		}
	   ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	    /*MAIN METHOD*/ 
	    public static void main(String[] args) {
	        //Schedule a job for the event-dispatching thread:
	        //creating and showing this application's GUI.
	        javax.swing.SwingUtilities.invokeLater(new Runnable() {
	            public void run() {
	                createAndShowGUI();
	            }
	        });
	        
	        
	    }//Main		

		
}//Class//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 